window.Imported = window.Imported || {};
window.Imported.TLB_SKAUICompendium = true;

window.TLB = window.TLB || {};
TLB.SKAUICompendium = TLB.SKAUICompendium || {};

/*:
 * @author coffeenahc / Trihan
 *
 * @target MZ
 * @plugindesc This plugin modifies the compendium scene with the update UI.
 * Commissioned work by coffeenahc for Star Knightness Aura.
 * @url https://gitgud.io/auragamedev/auramz
 *
 * @help
 * Use SceneManager.push(Scene_Compendium) to access.
 * 
 * After:
 * - TLB_SKABattleHUD 
 * - TLB_SKAElementDrain
 *
 * ============================================================================
 * Terms of Use
 * ============================================================================
 *
 * Copyright 2024 Auradev
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 * @param drainString_cte
 * @text Drain String
 * @desc The string to use when an enemy drains an element.
 * @default DRAIN
 * 
 * @param weakString_cte
 * @text Weak String
 * @desc The string to use when an enemy is weak against an element.
 * @default WEAK
 * 
 * @param resistString_cte
 * @text Resist String
 * @desc The string to use when an enemy resists an element.
 * @default RESIST
 *
 * @param categoryWindow
 * @text Category Window
 * 
 * @param noInfoClear
 * @parent categoryWindow
 * @text No Info Clear
 * @type string[]
 * @desc A list of category symbols that won't call clearInfo() on category select.
 * @default ["social", "locations"]
 *
 * @param category_bg_main
 * @parent categoryWindow
 * @text Category Main Background
 * @desc The image to use as the main category background.
 * @type file
 * @dir img/menu/
 * @default Item_Menu/ITEM_MENU_CATEGORY_BG_MAIN
 *
 * @param category_bg
 * @parent categoryWindow
 * @text Category Background
 * @desc The image to use as the category background.
 * @type file
 * @dir img/menu/
 * @default Item_Menu/ITEM_MENU_CATEGORY_BG
 *
 * @param categoryWindowX
 * @parent categoryWindow
 * @text X
 * @desc X coordinate of the category window rect.
 * @type number
 * @min -9999
 * @max 9999
 * @default -142
 *
 * @param categoryWindowY
 * @parent categoryWindow
 * @text Y
 * @desc Y coordinate of the category window rect.
 * @type number
 * @default 19
 *
 * @param categoryWindowWidth
 * @parent categoryWindow
 * @text Width
 * @desc Width of the category window rect.
 * @type number
 * @default 329
 *
 * @param categoryWindowHeight
 * @parent categoryWindow
 * @text Height
 * @desc Height of hte category window rect.
 * @type number
 * @default 270
 *
 * @param subcategoryWindow
 * @text Subcategory Window
 *
 * @param subcategory_bg
 * @parent subcategoryWindow
 * @text Subcategory Background
 * @desc The image to use as the subcategory background.
 * @type file
 * @dir img/menu/
 * @default Compendium/COMPENDIUM_SUBCATEGORY_WINDOW
 *
 * @param subcategory_cursor
 * @parent subcategoryWindow
 * @text Subcategory Cursor
 * @desc The image to use as the subcategory cursor.
 * @type file
 * @dir img/menu/
 * @default Quest_Menu/QUEST_MENU_SELECTION
 *
 * @param subcategoryWindowXOffset
 * @parent subcategoryWindow
 * @text X Offset
 * @desc X offset from the right of hte category window.
 * @type number
 * @default 5
 *
 * @param subcategoryWindowY
 * @parent subcategoryWindow
 * @text Y
 * @desc Y coordinate of the subcategory window rect.
 * @type number
 * @default 23
 *
 * @param subcategoryWindowWidth
 * @parent subcategoryWindow
 * @text Width
 * @desc Width of the subcategory window rect.
 * @type number
 * @default 256
 *
 * @param subcategoryWindowHeight
 * @parent subcategoryWindow
 * @text Height
 * @desc Height of the subcategory window rect.
 * @type number
 * @default 605
 *
 * @param informationWindow
 * @text Information Window
 *
 * @param information_bg
 * @parent informationWindow
 * @text Information Background
 * @desc The image to use as the information window background.
 * @type file
 * @dir img/menu/
 * @default Compendium/COMPENDIUM_INFORMATION_WINDOW
 *
 * @param informationWindowXOffset
 * @parent informationWindow
 * @text X Offset
 * @desc X offset from the right of the subcategory window.
 * @type number
 * @default 3
 *
 * @param informationWindowY
 * @parent informationWindow
 * @text Y
 * @desc Y coordinate of the information window rect.
 * @type number
 * @default 18
 *
 * @param informationWindowWidth
 * @parent informationWindow
 * @text Width
 * @desc Width of the information window rect.
 * @type number
 * @default 558
 *
 * @param informationWindowHeight
 * @parent informationWindow
 * @text Height
 * @desc Height of the information window rect.
 * @type number
 * @default 612
 *
 */

window.parameters = PluginManager.parameters('TLB_SKAUICompendium');
TLB.Param = TLB.Param || {};
TLB.Param.SKAUIC = TLB.Param.SKAUIC || {};

TLB.SKABase.parseParameters(parameters, TLB.Param.SKAUIC);

Object.defineProperties(TLB.SKAUICompendium, {
	"drainString": { get: () => TextManager.message(`auramzui/TLB_SKAUICompendium-drainString_cte`) || TLB.Param.SKAUIC.drainString_cte },
	"weakString": { get: () => TextManager.message(`auramzui/TLB_SKAUICompendium-weakString_cte`) || TLB.Param.SKAUIC.weakString_cte },
	"resistString": { get: () => TextManager.message(`auramzui/TLB_SKAUICompendium-resistString_cte`) || TLB.Param.SKAUIC.resistString_cte }
});

Scene_Compendium.prototype.categoryWindowRect = function() {
	const params = TLB.Param.SKAUIC;
	const wx = params.categoryWindowX;
	const wy = params.categoryWindowY;
	const ww = params.categoryWindowWidth;
	const wh = params.categoryWindowHeight;
	return new Rectangle(wx, wy, ww, wh);
};

Scene_Compendium.prototype.pageWindowRect = function() {
	const params = TLB.Param.SKAUIC;
	const wx = this._categoryWindow.x + this._categoryWindow.width + params.subcategoryWindowXOffset;
	const wy = params.subcategoryWindowY;
	const ww = params.subcategoryWindowWidth;
	const wh = params.subcategoryWindowHeight;
	return new Rectangle(wx, wy, ww, wh);
};

Scene_Compendium.prototype.detailWindowRect = function(parent) {
	const params = TLB.Param.SKAUIC;
	const wx = parent.x + parent.width + params.informationWindowXOffset;
	const wy = params.informationWindowY;
	const ww = params.informationWindowWidth;
	const wh = params.informationWindowHeight;
	return new Rectangle(wx, wy, ww, wh);
};

if (Imported.TLB_SKABattleHUD && Imported.TLB_SKAElementDrain) {
	const _Window_BattleEnemy_drawItem = Window_BattleEnemy.prototype.drawItem;
	Window_BattleEnemy.prototype.drawItem = function(index) {
		_Window_BattleEnemy_drawItem.call(this, index);
		const enemy = this._enemies[index];
		const rect = this.itemLineRect(index);
		rect.x += 54;
		rect.y += 40;
		const textOptions = {
			outlineGradient: ["#4f4f4f", "#000000"],
			outlineThickness: 2,
			dropShadow: true,
			dropShadowX: 0,
			dropShadowY: 1,
			shadowOpacity: 0.75
		};
		const gradient = ["#d9c5dd", "#eee5f1", "#d9c4de"];
		const action = BattleManager.inputtingAction();
		const element = action?.item().damage.elementId;
		const isAffinityKnown = BeastiaryManager.isKnownAffinity(enemy.enemyId(), element);
		if (isAffinityKnown) {
			let text = "";
			const elementId = TLB.SKAElementDrain.getDrainElement(enemy.enemy());
			if (elementId) {
				if (elementId === action.item().damage.elementId) {
					this.contents.fontFace = 'franklin-gothic-med';
					this.contents.fontSize = 18;
					const gradient = ["#d9c5dd", "#eee5f1", "#d9c4de"];
					const textOptions = {
						outlineGradient: ["#4f4f4f", "#000000"],
						outlineThickness: 2,
						dropShadow: true,
						dropShadowX: 0,
						dropShadowY: 1,
						shadowOpacity: 0.75
					};
					this.drawGradientText(TLB.SKAUICompendium.drainString, gradient, rect.x + 164, rect.y, 116, "center", textOptions);
				}
			}

			if (action.calcElementRate(enemy) > 1) text = TLB.SKAUICompendium.weakString;
			else if (action.calcElementRate(enemy) < 1) text = TLB.SKAUICompendium.resistString;
			this.drawGradientText(text, gradient, rect.x + 164, rect.y, 116, "center", textOptions);
		}
	};
}

(() => {
	const _Window_CompendiumCategory_initialize = Window_CompendiumCategory.prototype.initialize;
	Window_CompendiumCategory.prototype.initialize = function(rect) {
		this._itemBGSprites = [];
		_Window_CompendiumCategory_initialize.call(this, rect);
		this.height = this.maxItems() * this.itemHeight() + this.itemHeight();
		this.createContents();
		this.cursorVisible = false;
		this.opacity = 0;
		this.select(0);
		this.refresh();
	};
	
	Window_CompendiumCategory.prototype.maxCols = function() {
		return 1;
	};
	
	Window_CompendiumCategory.prototype.itemHeight = function() {
		return 79;
	};
	
	Window_CompendiumCategory.prototype.getIcons = function() {
		return [193, 192, 226];
	};
	
	Window_CompendiumCategory.prototype.drawAllItems = function() {
		const topIndex = this.topIndex();
		const icons = this.getIcons();
		for (let i = 0; i < this.maxVisibleItems(); i++) {
			const index = topIndex + i;
			if (index < this.maxItems()) {
				this.drawItemBackground(index);
				this.drawItem(index, icons[i]);
			}
		}
	};
	
	Window_CompendiumCategory.prototype.drawItemBackground = function(index) {
		const rect = this.itemRect(index);
		if (index > 1) rect.y -= 9 * (index - 1);
		let item = this._itemBGSprites[index];
		if (!item) {
			item = new Sprite();
			item.x = rect.x;
			item.y = rect.y;
			const image = index === 0 ? TLB.Param.SKAUIC.category_bg_main : TLB.Param.SKAUIC.category_bg;
			item.bitmap = ImageManager.loadMenu(image);
			this.addChildAt(item, 0);
			this._itemBGSprites[index] = item;
		}
	};
	
	Window_CompendiumCategory.prototype.drawItem = function(index, icon) {
		const rect = this.itemRect(index);
		const y = rect.y - 9 * (index - 2);
		this.drawIcon(icon, rect.x + 7, y);
		this.contents.fontFace = 'franklin-gothic-demi-cond';
		this.contents.fontSize = 30;
		this.drawGradientText(this.commandName(index), ["#d9c4de", "#eee5f1", "#d9c5dd"], rect.x + 45, y - 4, 180, "left", { outlineThickness: 3 });
	};
	
	Window_CompendiumCategory.prototype.refreshCursor = function() {
		const index = this.index();
		if (index >= 0) {
			const rect = this.itemRect(index);
			rect.x += 2;
			rect.y += 13 - (9 * index);
			this.setCursorRect(rect.x, rect.y, rect.width, rect.height);
			this.cursorVisible = true;
		} else {
			this.setCursorRect(0, 0, 0, 0);
			this.cursorVisible = false;
		}
	};
	
	Window_CompendiumCategory.prototype._createCursorSprite = function() {
		this._cursorSprite = new Sprite();
		const image = TLB.Param.SKAUIB.categorywindow_cursorimage;
		const bmp = ImageManager.loadMenu(image);
		this._cursorSprite.bitmap = bmp;
		this._clientArea.addChild(this._cursorSprite);
	};
	
	Window_CompendiumCategory.prototype._refreshCursor = function() {
		//
	};
	
	const _Window_Compendium_initialize = Window_Compendium.prototype.initialize;
	Window_Compendium.prototype.initialize = function(rect) {
		_Window_Compendium_initialize.call(this, rect);
		this.drawWindowBackground();
		const image = TLB.Param.SKAUIB.arrowimage;
		const bitmap = ImageManager.loadMenu(image);
		this._downArrowSprite.bitmap = bitmap;
		this._downArrowSprite.anchor.x = 0.5;
		this._downArrowSprite.anchor.y = 0.5;
		this._downArrowSprite.move(258 / 2, 632 - 15);
		this._upArrowSprite.bitmap = bitmap;
		this._upArrowSprite.anchor.x = 0.5;
		this._upArrowSprite.anchor.y = 0.5;
		this._upArrowSprite.scale.y = -1;
		this._upArrowSprite.move(258 / 2, 5);
		this.cursorVisible = false;
		this._textOptions = { outlineThickness: 3 };
		this._standardGradient = ["#d9c4de", "#eee5f1", "#d9c5dd"];
		this.opacity = 0;
	};
	
	Window_Compendium.prototype.maxScrollY = function() {
		return Math.max(0, this.overallHeight() - this.innerHeight + this.itemHeight() / 2);
	};
	
	Window_Compendium.prototype.ensureCursorVisible = function(smooth) {
		if (this._cursorAll) {
			this.scrollTo(0, 0);
		} else if (this.innerHeight > 0 && this.row() >= 0) {
			const scrollY = this.scrollY();
			const itemTop = this.row() * this.itemHeight();
			const itemBottom = itemTop + this.itemHeight();
			const scrollMin = itemBottom - this.innerHeight + this.itemHeight() / 2;
			if (scrollY > itemTop) {
				if (smooth) {
					this.smoothScrollTo(0, itemTop);
				} else {
					this.scrollTo(0, itemTop);
				}
			} else if (scrollY < scrollMin) {
				if (smooth) {
					this.smoothScrollTo(0, scrollMin);
				} else {
					this.scrollTo(0, scrollMin);
				}
			}
		}	
	};
	
	Window_Compendium.prototype.drawWindowBackground = function() {
		this._windowBG = new Sprite();
		this._windowBG.bitmap = ImageManager.loadMenu(TLB.Param.SKAUIC.subcategory_bg);
		this._windowBG.y -= 7;
		this.addChildAt(this._windowBG, 0);	
	};
	
	Window_Compendium.prototype.itemRect = function(index) {
		const rect = Window_Selectable.prototype.itemRect.call(this, index);
		return new Rectangle(rect.x + 2, rect.y + 15, rect.width - 4, rect.height);
	};
	
	Window_Compendium.prototype.drawItemBackground = function(index) {
		const rect = this.itemRect(index);
		this.drawBackgroundRect(rect, index);		
	};
	
	Window_Compendium.prototype.drawBackgroundRect = function(rect, index) {
		let c1 = "";
		let c2 = "";
		if (index % 2 === 0) {
			c1 = "#373845";
			c2 = "#353441"
		} else {
			c1 = "#474853";
			c2 = "#4b4957"
		}
		if (index == 0) {
			c1 = "#302f3c";
		}
		const x = rect.x - 3;
		const y = rect.y - 7;
		const w = rect.width + 7;
		const h = rect.height + 2;
		this.contentsBack.gradientFillRect(x, y, w, h, c1, c2, true);
		this.contentsBack.strokeRect(x, y, w, h, c1);
	};
	
	Window_Compendium.prototype.refreshCursor = function() {
		const index = this.index();
		if (index >= 0) {
			const rect = this.itemRect(index);
			rect.x -= 6;
			rect.y -= 7;
			this.setCursorRect(rect.x, rect.y, rect.width, rect.height);
			this.cursorVisible = true;
		} else {
			this.setCursorRect(0, -40, 0, 0);
			this.cursorVisible = false;
		}
	};
	
	Window_Compendium.prototype.itemHeight = function() {
		return 44;
	};
	
	Window_Compendium.prototype._createCursorSprite = function() {
		this._cursorSprite = new Sprite();
		const image = TLB.Param.SKAUIC.subcategory_cursor;
		this._cursorSprite.bitmap = ImageManager.loadMenu(image);
		this._clientArea.addChild(this._cursorSprite);
	};
	
	Window_Compendium.prototype._refreshCursor = function() {
		//
	};
	
	Window_Compendium.prototype._refreshArrows = function() {
		//
	};
	
	Window_Compendium.prototype.updateArrows = function() {
		this.downArrowVisible = this.maxItems() - this.topIndex() > 13;
		this.upArrowVisible = this.topIndex() > 0;
	};
	
	Window_CompendiumDetail.prototype.initialize = function(rect) {
		Window_Selectable.prototype.initialize.call(this, rect);
		this.drawWindowBackground();
		this.deactivate();
		this._standardGradient = ["#d9c4de", "#eee5f1", "#d9c5dd"];
		this._specialGradient = ["#dcb38d", "#ca9a91", "#bc8694"];
		this.select(-1);
		this.opacity = 0;
	};
	
	Window_CompendiumDetail.prototype.drawWindowBackground = function() {
		this._windowBG = new Sprite();
		this._windowBG.bitmap = ImageManager.loadMenu(TLB.Param.SKAUIC.information_bg);
		this.addChildAt(this._windowBG, 0);		
	};
	
	Window_CompendiumDetail.prototype.maxItems = function() {
		return this._data ? this._data.length : 0;
	};
	
	Window_CompendiumDetail.prototype.itemRect = function(index) {
		const rect = Window_Selectable.prototype.itemRect.call(this, index);
		rect.y += 10;
		return rect;
	};
	
	Window_CompendiumDetail.prototype.maxPageRows = function() {
		return 7;
	};
	
	Window_CompendiumDetail.prototype.itemHeight = function() {
		return 82;
	};
	
	Window_EnemyList.prototype.drawItem = function(index) {
		const item = this.itemAt(index);
		const rect = this.itemLineRect(index);
		rect.y -= 9;
		this.contents.fontFace = 'franklin-gothic-med';
		this.contents.fontSize = 22;
		if (typeof item == "string") {
			const symbol = this.isExpanded(item) ? "-" : "+";
			this.drawGradientText(`${symbol} ${item}`, this._standardGradient, rect.x, rect.y, rect.width, "center", this._textOptions);
		} else {
			this.contents.fontSize = 16;
			this.drawGradientText(item.name, ["#ad9cb1", "#c3bbc5", "#b0a0b4"], rect.x, rect.y, rect.width, "center", this._textOptions);
		}
	};
	
	Window_EnemyDetail.prototype.drawAffinities = function() {
		const mapElementIDToIconID = [null, 77, 64, null, 66, 67, 68, 69, 70, 71];

		let row = 0;
		const rect = this.itemRect(0);
		for (let i = 0; i < $dataSystem.elements.length; i++) {
			const x = rect.width - rect.width / 4 + (i < 6 ? 30 : 80);
			const y = rect.y + row * 58 + 18;
			const iconID = mapElementIDToIconID[i];
			if (iconID != null) {
				this.drawIcon(iconID, x, y);
				const affinity = Window_EnemyDetail.prototype.getAffinity.call(this, i);
				this.drawGradientText(affinity, this._standardGradient, x, y + ImageManager.iconWidth - 4, ImageManager.iconWidth, "center", this._textOptions);
				if (row === 3) { row = 0; }
				else { ++row; }
			}
		}
	};
	
	Window_EnemyDetail.prototype.drawStats = function() {
		const statTxt = ["#d4aae5", "#c7ade5", "#b7b0e6"];
		const stats = Window_EnemyDetail.prototype.makeStats.call(this);
		for (let i = 0; i < stats.length; i++) {
			const stat = stats[i];
			const rect = this.itemRect(i);
			const y = rect.y - i * 50 + 10;
			this.drawGradientText(stat.name, statTxt, rect.width / 2, y, rect.width / 4, "left", this._textOptions);
			this.drawGradientText(stat.value, this._standardGradient, rect.width / 2, y, rect.width / 4, "right", this._textOptions);
		}		
	};
	
	Window_EnemyDetail.prototype.drawSkills = function() {
		const skills = Window_EnemyDetail.prototype.makeSkills.call(this);
		this.contents.fontSize = 25;
		this.drawGradientText(AuraMZ.Compendium.skillsText, this._specialGradient, 10, 280, 400, "left", this._textOptions);
		this.contents.fontSize = 22;
		const rect = this.itemRect(0);
		for (let i = 0; i < skills.length; i++) {
			const y = 290 + ((i + 1) * 30);
			const skill = skills[i];
			if (BeastiaryManager.isKnownSkill(this._enemy.id, skill.id)) {
				this.drawIcon(skill.iconIndex, 10, y + 2);
				this.drawGradientText(skill.name, this._standardGradient, 13 + ImageManager.iconWidth, y - 2, rect.width / 2 - 48, "left", this._textOptions);
			} else {
				this.drawGradientText("?", this._standardGradient, 10, y, rect.width / 2 - 48, "left", this._textOptions);
			}
		}		
	};
	
	Window_EnemyDetail.prototype.drawTraits = function() {
		const traits = Window_EnemyDetail.prototype.makeTraits.call(this);
		this.contents.fontSize = 25;
		this.drawGradientText(AuraMZ.Compendium.traitsText, this._specialGradient, this.width / 2 - 15, 280, 400, "left", this._textOptions);
		this.contents.fontSize = 22;
		const rect = this.itemRect(0);
		for (let i = 0; i < traits.length; i++) {
			const y = 290 + ((i + 1) * 30);
			this.drawGradientText(traits[i], this._standardGradient, this.width / 2 - 15, y, rect.width, "left", this._textOptions);
		}		
	};
	
	Window_EnemyDetail.prototype.drawKilledBosses = function() {
		const text = this.makeKilledBosses();
		if (text) {
			this.contents.fontSize = 25;
			this.drawGradientText(AuraMZ.Compendium.defeatedText, this._specialGradient, 10 + this.width / 2, this.height - 70, this.width, "left", this._textOptions);
			this.contents.fontSize = 22;
			this.drawGradientText(`${text[0]}/${text[1]}`, this._standardGradient, this.width / 2, this.height - 70, this.width / 2 - 50, "right", this._textOptions);
		}		
	}
	
	Window_SocialList.prototype.drawItem = function(index) {
		const item = this.itemAt(index);
		const rect = this.itemLineRect(index);
		rect.y -= 9;
		this.contents.fontFace = 'franklin-gothic-med';
		this.contents.fontSize = 22;
		this.drawGradientText(item.name_cte, this._standardGradient, rect.x, rect.y, rect.width, "center", this._textOptions);
	};
	
	Window_SocialDetail.prototype.maxScrollY = function() {
		return Window_CompendiumDetail.prototype.maxScrollY.call(this) + (this._data.length > 7 ? 20 : 0);
	}
	
	Window_SocialDetail.prototype.drawItem = function(index) {
		const specialTxt = ["#6b74a5", "#888fbe", "#a3aad9"];
		const item = this.itemAt(index);
		const rect = this.itemRect(index);
		const bitmap = ImageManager.loadCharacter(item.characterName);
		if (bitmap && !bitmap.isReady()) {
			bitmap.addLoadListener(this.drawCharacterWithPattern.bind(this, item.characterName, item.characterIndex, rect.x + 40, rect.y + 60, item.direction, item.pattern));
		} else {
			this.drawCharacterWithPattern(item.characterName, item.characterIndex, rect.x + 40, rect.y + 60, item.direction, item.pattern);
		}
		this.contents.fontFace = "franklin-gothic-med-cond";
		this.contents.fontSize = 23;
		this.drawGradientText(item.name, specialTxt, rect.x + 70, rect.y + 10, rect.width, "left", this._textOptions);
		this.contents.fontSize = 20;
		this.drawGradientText(item.tags.map(tag => eval(tag.text_cte)).join(", "), this._standardGradient, rect.x + 80, rect.y + 30, rect.width - 20, "left", this._textOptions);
	};
	
	Window_LocationList.prototype.drawItem = function(index) {
		const item = this.itemAt(index);
		const locationItem = $dataCompendium.locations.find(loc => loc.name_cte == item.name_cte);
		const rect = this.itemLineRect(index);
		Window_LocationList.prototype.drawShrine.call(this, locationItem, 5, rect.y);
		rect.x += 10;
		rect.y -= 9;
		this.contents.fontFace = 'franklin-gothic-med';
		this.contents.fontSize = 22;
		this.drawGradientText(item.name_cte, this._standardGradient, rect.x, rect.y, rect.width, "center", this._textOptions);
	};
	
	const _Window_LocationDetail_initialize = Window_LocationDetail.prototype.initialize;
	Window_LocationDetail.prototype.initialize = function(rect) {
		_Window_LocationDetail_initialize.call(this, rect);
		this.drawWindowBackground();
		this.deactivate();
		this._standardGradient = ["#d9c4de", "#eee5f1", "#d9c5dd"];
		this._specialGradient = ["#dcb38d", "#ca9a91", "#bc8694"];
		this.select(-1);
		this.opacity = 0;
	};
	
	Window_LocationDetail.prototype.drawWindowBackground = Window_CompendiumDetail.prototype.drawWindowBackground;
	
	Window_LocationDetail.prototype.maxCols = function() {
		return 2;
	};
	
	Window_LocationDetail.prototype.itemRect = function(index) {
		const rect = Window_HorzCommand.prototype.itemRect.call(this, index);
		rect.y += 510;
		return rect;
	};
	
	const _Window_LocationDetail_drawThumbnail = Window_LocationDetail.prototype.drawThumbnail;
	Window_LocationDetail.prototype.drawThumbnail = function() {
		_Window_LocationDetail_drawThumbnail.call(this);
		if (this._worldmapThumbnail) {
			this._worldmapThumbnail.filterArea = new Rectangle(800, 60, 332, 188);
		}
	};
	
	Window_LocationDetail.prototype.drawDescription = function() {
		const location = this._location;
		const locIndex = $dataCompendium.locations.findIndex(loc => loc == location);
		let paraY = 220;
		for (let i = 0; i < (location.descriptions || []).length; i++) {
			const paragraph = location.descriptions[i];
			if (!paragraph.condition || eval(paragraph.condition)) {
				this.resetTextColor();
				this.changePaintOpacity(true);
				this.contents.fontSize = 18;
				this.contents.fontFace = "franklin-gothic-med-cond";
				for (let j = 0; j < paragraph.description_cte.length; j++) {
					const line = paragraph.description_cte[j];
					this.drawTextEx(window.CTE ? TextManager.localize(line, "compendium", "locations", locIndex, "descriptions", i, "description_cte", j) : line, 10, paraY, this.contentsWidth(), true);
					paraY += this.textSizeEx(line, true).height;
				}
			}
		}
	};
})();