//=============================================================================
// RPG Maker MZ - Steam Deploy
// ----------------------------------------------------------------------------
// (C)2025 aura-dev
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// Version
// 1.0.0 2025/03/27
//=============================================================================

/*:
 * @target MZ
 * @plugindesc Steam Deploy
 * @author aura-dev
 * @url https://gitgud.io/auragamedev/auramz
 *
 * @help steam.js
 *
 * Steam deploy specific customizations.
 *
 * @param steamMode
 * @type boolean
 * @text Steam Mode
 * @desc Whether the game is in steam mode or not
 * @default false
 * 
 * @param onUnlockAchievementCommonEventID
 * @type number
 * @text On Unlock Achievement CommonEvent ID
 * @desc Common Event that gets called whenever an achievement is unlocked
 * @default 729
 * 
 * @param altDisclaimer_cte
 * @desc Alternate Disclaimer text
 * @text Alt Disclaimer Text
 * @type string
 * @default This is a work of fiction.\nAll characters depicted are at least 18 years of age.
 * 
 * @param adultContentSwitch
 * @type switch
 * @text Adult Content Switch
 * @desc Will hard set the switch to false when doing a readAdultContent command
 * @default 9
 *
 * @command activateAchievement
 * @text Activate Achievement
 * @desc Activates the specified achievement for the player.
 *
 * @arg achievementName
 * @type text
 * @text Achievement Name
 * @desc The name of the achievement to activate.
 */

window.AuraMZ = window.AuraMZ || {};
AuraMZ.Steam = {};

(($) => {
	const PLUGIN_ID = "ska_steam";
	const PARAMS = PluginManager.parameters(PLUGIN_ID);
	$.steamMode = PARAMS["steamMode"] === 'true';
	$.adultContentSwitch = parseInt(PARAMS["adultContentSwitch"]);
	$.onUnlockAchievementCommonEventID = parseInt(PARAMS["onUnlockAchievementCommonEventID"]);

	Object.defineProperties($, {
		"alt_disclaimer_text": { get: () => TextManager.message(`auramz/${PLUGIN_NAME}-altDisclaimer_cte`) || PARAMS.altDisclaimer_cte }
	});

	PluginManager.registerCommand(PLUGIN_ID, 'activateAchievement', args => {
		if ($.steamMode) {
			const achievementName = String(args.achievementName);
			$.activateAchievement(achievementName);
		} else {
			console.log("Steam mode not enabled");
		}
	});

	if ($.steamMode) {
		if (AuraMZ.disclaimer_text) {
			AuraMZ.disclaimer_text = $.alt_disclaimer_text;
		}
		
		if (GBC?.Param?.Title?.display_icon_links) {
			GBC.Param.Title.display_icon_links = false;
		}
		
		if (!PluginManager._commands["adult_content_option:readAdultOption"]) {
			PluginManager.registerCommand("adult_content_option", "readAdultOption", () => {
				$gameSwitches.setValue($.adultContentSwitch, false);
			});
		}

		// Overwrite loadImages function to turn off trying to prefetch removed images
		Game_Interpreter.prototype.loadImages = function() {
			const list = this._list.slice(0, 200);
			for (const command of list) {
				switch (command.code) {
					case 101: // Show Text
						ImageManager.loadFace(command.parameters[0]);
						break;
					case 231: // Show Picture
						const pictureName = command.parameters[1];
						const isCensoredPicture = pictureName.startsWith("hcg/") || (pictureName.startsWith("si/") && pictureName.contains("Pose_"))
						if (!isCensoredPicture) {
							ImageManager.loadPicture(pictureName);
						}
						break;
				}
			}
		};

		$.activateAchievement = function(achievementId) {
			SteamLink.getAchievement(achievementId)
				.then((is_achieved) => {
					if (!is_achieved) {
						SteamLink.activateAchievement(achievementId);
						$gameTemp.reserveCommonEvent($.onUnlockAchievementCommonEventID);
					}
				}, (_) => {
					console.log(`Failed to check achievement: ${achievementId}`);
				})
				.catch(e => {
					console.log(e)
				});
		}
	}
})(AuraMZ.Steam);