//=============================================================================
// RPG Maker MZ - Liora Battle
// ----------------------------------------------------------------------------
// (C)2025 aura-dev
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// Version
// 1.0.0 02/04/2025
//=============================================================================

/*:
 * @target MZ
 * @plugindesc Liora Battle Customizations
 * @author aura-dev
 * @url https://gitgud.io/auragamedev/auramz
 *
 * @help liora_battle.js
 *
 * Contains game customizations for Liora specific battle rules.
 * 
 * @param gainsText_cte
 * @text Gains Text
 * @desc Text to use for "gains" game messages
 * @default %1 gained %2!
 * 
 */

window.Liora = window.Liora || {};
Liora.Battle = {};

(($) => {
	const PLUGIN_ID = "liora_battle";
	$.Params = PluginManager.parameters(PLUGIN_ID);
	Object.defineProperties($, {
		"gainsText": { get: () => TextManager.message(`${PLUGIN_ID}-gainsText_cte`) || $.Params.gainsText_cte },
	});

	// Creeate initial area level data
	$.areaVariables = { 
		13: 221,  // Castle
		10: 222, 25: 222, 26: 222, 27: 222, // Forest
		28: 222 // Intro
	};
	$.mapBaseLevels = { 
		13: 10, // Castle
		10: 1, 25: 1, 26: 1, 27: 2,  // Forest
		28: 1 // Intro
	};
	
	$.mapBaseLevel = function(mapId) {
		return $.mapBaseLevels[mapId];
	}

	// Lookip area level and initialize it if it's not defined
	$.areaLevel = function(mapId) {
		const areaLevelVariable = $.areaVariables[mapId];
		return $gameVariables.value(areaLevelVariable) + $.mapBaseLevel(mapId);
	}
	
	// Returns the main dungeon area
	$.areaMainVariable = function() {
		return $.areaVariables[13];
	}
	
	$.areaMainLevel = function() {
		return $.areaLevel(13);
	}

	// Decrease level of main area and increase area of target area
	$.powerShift = function(targetArea) {
		const shift = 1;
		const mainVariable = $.areaMainVariable();
		const targetVariable = $.areaVariables[targetArea];
		$gameVariables.setValue(mainVariable, $gameVariables.value(mainVariable) - shift);
		$gameVariables.setValue(targetVariable, $gameVariables.value(targetVariable) + shift);
	}

	Game_Enemy.prototype.currentClass = function() {
		const classId = eval(this.enemy().meta.class);
		return $dataClasses[classId];
	};

	Game_Enemy.prototype.level = function() {
		return eval(this.enemy().meta.level);
	};

	// Inject logic to add class stats to params
	$._Game_Enemy_paramBase = Game_Enemy.prototype.paramBase;
	Game_Enemy.prototype.paramBase = function(paramId) {
		const base = $._Game_Enemy_paramBase.call(this, paramId);
		return base + this.currentClass().params[paramId][this.level()];
	};

	// Inject logic to add class to trait objects
	$._Game_Enemy_traitObjects = Game_Enemy.prototype.traitObjects;
	Game_Enemy.prototype.traitObjects = function() {
		const objects = $._Game_Enemy_traitObjects.call(this);
		objects.push(this.currentClass());
		return objects;
	};

	// Adds Wounded passive to all KOed members
	$.processWounds = function() {
		let gainedWound = false;
		for (const member of $gameParty.members()) {
			if (!member.isAlive()) {
				if (member.increaseSkillRank(48)) {
					const extraWoundsTraits = member.traitObjects().filter(traitObject => traitObject.meta.extra_wounds);
					const extraWounds = extraWoundsTraits.reduce((r, traitObject) => r + eval(traitObject.meta.extra_wounds), 0);
					for (let i = 0; i < extraWounds; ++i) {
						member.increaseSkillRank(48);
					}
					gainedWound = true;
					const wound = $dataSkills[48 + member.getSkillRank(48)];
					const text = $.gainsText.format(member.name(), wound.name);
					$gameMessage.add("\\." + text);
				}

			}
		}

		if (gainedWound) {
			$gameMessage.newPage();
		}
	}

	// Inject logic to check for wounding
	$._BattleManager_processAbort = BattleManager.processAbort;
	BattleManager.processAbort = function() {
		$.processWounds();
		$._BattleManager_processAbort.call(this);
	};

	// Inject logic to check for wounding
	$._BattleManager_processVictory = BattleManager.processVictory;
	BattleManager.processVictory = function() {
		$.processWounds();
		$._BattleManager_processVictory.call(this);
	};
})(Liora.Battle);

