import json
import os
import shutil
import sys
from utils.config import DEFAULT_LANGUAGE, LANGUAGES_FILE
from utils.json_helper import clear_values

def main():
	print("Updating translation files...")
	with open(LANGUAGES_FILE, "r", encoding="utf-8") as f:
		trans_dirs = json.load(f)
			
	for root, _, files in os.walk(DEFAULT_LANGUAGE):
		for file in files:
			if file.endswith(".json"):
				file_path = os.path.join(root, file)
				with open(file_path, "r", encoding="utf-8") as f:
					try:
						en_data = json.load(f)
						for dir in trans_dirs:
							if dir == DEFAULT_LANGUAGE: continue
							trans_file_path = os.path.join(root.replace(DEFAULT_LANGUAGE, dir), file)
							if not os.path.isfile(trans_file_path):
								with open(trans_file_path, "w") as new_file:
									print(f"Creating missing file {trans_file_path}")
									new_file.write("{}")
									new_file.close()
							with open(trans_file_path, "r", encoding="utf-8") as f2:
								trans_data = json.load(f2)
								modified = False
									
								def update_json(en, tr):
									modified = False
									for key, value_en in en.items():
										if key not in tr:
											print(f"Adding missing key {key} to {trans_file_path}")
											modified = True
											if isinstance(value_en, dict):
												tr[key] = clear_values(value_en)
											else:
												tr[key] = ""
										elif isinstance(value_en, dict) and isinstance(tr[key], dict):
											modified |= update_json(value_en, tr[key])

									return modified
								
								def sort_keys(en, tr):
									for k, v in en.items():
										if isinstance(tr[k], dict):
											tr[k] = sort_keys(en[k], tr[k])
										else:
											sorted_data_keys = json.dumps({k: tr[k] for k in en})
											tr = json.loads(sorted_data_keys)
									return tr
												
								modified = update_json(en_data, trans_data)
								sorted_data_keys = json.dumps({k: trans_data[k] for k in en_data})
								sorted_output = json.loads(sorted_data_keys)
								sorted_output = sort_keys(en_data, sorted_output)
								
								if modified:
									print(f"Writing changes to file: {trans_file_path}")
									with open(trans_file_path, "w", encoding="utf-8") as f_out:
										json.dump(sorted_output, f_out, ensure_ascii=False, indent=4)
								
					except json.JSONDecodeError:
						if trans_file_path:
							print(f"Warning: Could not parse {trans_file_path}")
						else:
							print(f"Warning: Could not parse {file_path}")
							
	print("Done!")
	
if __name__ == "__main__":
    main()